package Catalog;
use DBI;
use Item;
use strict;

sub new
{
    my $class = shift;
    my $item_id = shift;
    my $self = bless {
        handle => undef,
        state  => undef,
    }, $class;
    $self->_initialize($item_id);
    $self;
}


sub _initialize
{
    my $self = shift;
    my $item_id = shift;
    my $handle = $self->handle;
    eval {
        if (defined $item_id) { 
            $self->state(
                $handle->prepare(q{
                    SELECT id, name, price, photo, feature, size
                      FROM catalog
                     WHERE id = ?
                })
            );
            $self->state->execute($item_id);
        }
        else {
            $self->state(
                $handle->prepare(q{
                    SELECT id, name, price, photo, feature, size
                      FROM catalog
                })
            );
            $self->state->execute;
        }
    };
    if ($@) { die $@ }
    $self;
}


sub get_item
{
    my $self = shift;
    my $state = $self->state;
    return undef unless my $record = $state->fetchrow_arrayref;
    return Item->new(
        id      => $record->[0],
        name    => $record->[1],
        price   => $record->[2],
        photo   => $record->[3],
        feature => $record->[4],
        size    => $record->[5],
    );
}


sub close
{
    my $self = shift;
    $self->state->finish      if $self->state;
    $self->handle->disconnect if $self->handle;
}



sub handle
{
    my $self = shift;
    unless ($self->{handle}) {
        eval {
            $self->{handle} = DBI->connect(
                'dbi:mysql:webapp', 'root', 'test', {
                RaiseError => 1
            }) or die;
        };
        if ($@) { die "connect: $@" }
    }
    $self->{handle};
}


sub state
{
    my $self = shift;
    if (@_) { $self->{state} = shift }
    $self->{state};
}



sub DESTROY
{
    my $self = shift;
    $self->close;
}

1;
__END__
